#ifndef NexMegaIO_h
#define NexMegaIO_h

class NexMegaIO {
public:
  // Full constructor :
  NexMegaIO(HardwareSerial& uart, uint32_t bauds)
    : _bauds(bauds), NexPort(uart) {}
  // Default constructor :
  NexMegaIO()
    : NexMegaIO(Serial1, 115200) {}
  // To prevent early initialisation conflicts :
  void begin() {
    NexPort.begin(_bauds);
  }
  // Handling incoming bytes
  void process(uint8_t abyte) {
    if (abyte & 0x80) {    // it's a command byte
      if (abyte & 0x40) {  // it's a read command
        _readflag = true;
        _wait4data = 1;  // set the number of expected data bytes following
      } else {           // it's a write command
        _readflag = false;
        _wait4data = 2;  // set the number of expected data bytes following
      }
      _recdata = 0;                    // initialise the data pointer
      _netadr = (abyte & 0x38) >> 3;   // save the network address (actually unused, thus don't care)
      _type = abyte & 0x07;            // save the data type
    } else {                           // it's a data byte
      if (_wait4data > 0) {            // are we expecting data bytes ? If not, don't care
        _rawdata[_recdata++] = abyte;  // save the data byte and increment the pointer afterwards
        _wait4data--;                  // decrement the number of expected data bytes following
        if (_wait4data == 0) {         // we have received the expected number of data bytes and may now process
          _channel = (_rawdata[0] & 0x78) >> 3;
          if (_readflag) {  // handle the read request
            switch (_type) {
              case 0x02:                                                                 // PWM
                NexPort.write(0x82);                                                     // Command byte for write PWM
                NexPort.write((_channel << 3) + (_pwmValues[_channel] & 0x80 ? 1 : 0));  // First data byte
                NexPort.write(_pwmValues[_channel] & 0x7F);                              // Second data byte
                break;
              default:  // do nothing
                break;
            }
          } else {  // handle the write request
            switch (_type) {
              case 0x02:                                                                // PWM
                _pwmValues[_channel] = _rawdata[1] + ((_rawdata[0] & 0x01) ? 128 : 0);  // save the data for further read
                analogWrite(_channel + 2, _pwmValues[_channel]);                        // update the PWM pin
                break;
              default:  // do nothing
                break;
            }
          }
        }
      }
    }
  }

  HardwareSerial& NexPort;
private:
  uint32_t _bauds;
  uint8_t _rawdata[2];
  uint8_t _wait4data = 0;
  uint8_t _recdata = 0;
  uint8_t _netadr;
  uint8_t _type;
  uint8_t _channel;
  uint8_t _ddata;
  uint16_t _adata;
  uint8_t _pwmValues[12] = { 0 };
  bool _readflag;
};
#endif